#!/usr/bin/env python

"""
env
PYTHONPATH=/net/soft_scratch/users/jeremys/git/ocio.js/build/src/pyglue/
LD_LIBRARY_PATH=/net/soft_scratch/users/jeremys/git/ocio.js/build/src/core/
"""

import math, os, sys
import PyOpenColorIO as OCIO

print "OCIO",OCIO.version

outputfilename = "config.ocio"

def WriteSPI1D(filename, fromMin, fromMax, data):
    f = file(filename,'w')
    f.write("Version 1\n")
    f.write("From %s %s\n" % (fromMin, fromMax))
    f.write("Length %d\n" % len(data))
    f.write("Components 1\n")
    f.write("{\n")
    for value in data:
        f.write("        %s\n" % value)
    f.write("}\n")
    f.close()

def Fit(value, fromMin, fromMax, toMin, toMax):
    if fromMin == fromMax:
        raise ValueError("fromMin == fromMax")
    return (value - fromMin) / (fromMax - fromMin) * (toMax - toMin) + toMin


###############################################################################


config = OCIO.Config()
config.setSearchPath('luts')

config.setRole(OCIO.Constants.ROLE_SCENE_LINEAR, "linear")
config.setRole(OCIO.Constants.ROLE_REFERENCE, "linear")
config.setRole(OCIO.Constants.ROLE_COLOR_TIMING, "Cineon")
config.setRole(OCIO.Constants.ROLE_COMPOSITING_LOG, "Cineon")
config.setRole(OCIO.Constants.ROLE_DATA,"raw")
config.setRole(OCIO.Constants.ROLE_DEFAULT,"raw")
config.setRole(OCIO.Constants.ROLE_COLOR_PICKING,"sRGB")
config.setRole(OCIO.Constants.ROLE_MATTE_PAINT,"sRGB")
config.setRole(OCIO.Constants.ROLE_TEXTURE_PAINT,"sRGB")


###############################################################################

cs = OCIO.ColorSpace(name='linear')
cs.setDescription("Scene-linear, high dynamic range. Used for rendering and compositing.")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_LG2)
cs.setAllocationVars([-15.0, 6.0])
config.addColorSpace(cs)


###############################################################################

def toSRGB(v):
    if v<0.04045/12.92:
        return v*12.92
    return 1.055 * v**(1.0/2.4) - 0.055

def fromSRGB(v):
    if v<0.04045:
        return v/12.92
    return ((v + .055) / 1.055) ** 2.4

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromSRGB(x))

# Data is srgb->linear
WriteSPI1D('luts/srgb.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='sRGB')
cs.setDescription("Standard RGB Display Space")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('srgb.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


NUM_SAMPLES = 2**16+25
RANGE = (-0.125, 4.875)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromSRGB(x))

# Data is srgb->linear
WriteSPI1D('luts/srgbf.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='sRGBf')
cs.setDescription("Standard RGB Display Space, but with additional range to preserve float highlights.")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('srgbf.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################

def toRec709(v):
    if v<0.018:
        return v*4.5
    return 1.099 * v**0.45 - 0.099

def fromRec709(v):
    if v<0.018*4.5:
        return v/4.5
    return ((v + .099) / 1.099) ** (1.0/0.45)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromRec709(x))

# Data is srgb->linear
WriteSPI1D('luts/rec709.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='rec709')
cs.setDescription("Rec. 709 (Full Range) Display Space")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('rec709.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################

cineonBlackOffset = 10.0 ** ((95.0 - 685.0)/300.0)

def fromCineon(x):
    return (10.0**((1023.0 * x - 685.0) / 300.0) - cineonBlackOffset) / (1.0 - cineonBlackOffset)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromCineon(x))

# Data is srgb->linear
WriteSPI1D('luts/cineon.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='Cineon')
cs.setDescription("Cineon (Log Film Scan)")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('cineon.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)



###############################################################################

cs = OCIO.ColorSpace(name='Gamma1.8')
cs.setDescription("Emulates a idealized Gamma 1.8 display device.")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([0.0, 1.0])

t = OCIO.ExponentTransform(value=(1.8,1.8,1.8,1.0))
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)

cs = OCIO.ColorSpace(name='Gamma2.2')
cs.setDescription("Emulates a idealized Gamma 2.2 display device.")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([0.0, 1.0])

t = OCIO.ExponentTransform(value=(2.2,2.2,2.2,1.0))
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################


# Log to Linear light conversions for Panalog
# WARNING: these are estimations known to be close enough.
# The actual transfer functions are not published

panalogBlackOffset = 10.0 ** ((64.0 - 681.0) / 444.0)

def fromPanalog(x):
    return (10.0**((1023 * x - 681.0) / 444.0) - panalogBlackOffset) / (1.0 - panalogBlackOffset)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromPanalog(x))

# Data is srgb->linear
WriteSPI1D('luts/panalog.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='Panalog')
cs.setDescription("Sony/Panavision Genesis Log Space")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('panalog.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)



###############################################################################



redBlackOffset = 10.0 ** ((0.0 - 1023.0) / 511.0)

def fromREDLog(x):
    return ((10.0 ** ((1023.0 * x - 1023.0) / 511.0)) - redBlackOffset) / (1.0 - redBlackOffset)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings


NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromREDLog(x))

# Data is srgb->linear
WriteSPI1D('luts/redlog.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='REDLog')
cs.setDescription("RED Log Space")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('redlog.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)



###############################################################################

def fromViperLog(x):
    return 10.0**((1023.0 * x - 1023.0) / 500.0)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings


NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromViperLog(x))

# Data is srgb->linear
WriteSPI1D('luts/viperlog.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='ViperLog')
cs.setDescription("Viper Log Space")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('viperlog.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)



###############################################################################


alexav3logc_a = 5.555556
alexav3logc_b = 0.052272
alexav3logc_c = 0.247190
alexav3logc_d = 0.385537
alexav3logc_e = 5.367655
alexav3logc_f = 0.092809
alexav3logc_cut = 0.010591
alexav3logc_eCutF = alexav3logc_e*alexav3logc_cut + alexav3logc_f

# This corresponds to EI800 per Arri Doc
# http://www.arridigital.com/forum/index.php?topic=6372.0
# http://www.arri.com/?eID=registration&file_uid=7775
def fromAlexaV3LogC(x):
    if x > alexav3logc_eCutF:
        return (10.0 **((x - alexav3logc_d) / alexav3logc_c) - alexav3logc_b) / alexav3logc_a
    else:
        return (x - alexav3logc_f) / alexav3logc_e


# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings


NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromAlexaV3LogC(x))

# Data is srgb->linear
WriteSPI1D('luts/alexalogc.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='AlexaV3LogC')
cs.setDescription("Alexa Log C")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('alexalogc.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################

'PLogLin'

# Josh Pines style pivoted log/lin conversion
minLinValue = 1e-10
linReference = 0.18
logReference = 445.0
negativeGamma = 0.6
densityPerCodeValue = 0.002
ngOverDpcv = negativeGamma/densityPerCodeValue
dpcvOverNg = densityPerCodeValue/negativeGamma

def fromPLogLin(x):
    return (10.0**((x*1023.0 - logReference)*dpcvOverNg ) * linReference)


# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromPLogLin(x))

# Data is srgb->linear
WriteSPI1D('luts/ploglin.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='PLogLin')
cs.setDescription("Josh Pines style pivoted log/lin conversion. 445->0.18")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('ploglin.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################

def fromSLog(x):
    return (10.0 ** (((x - 0.616596 - 0.03) / 0.432699)) - 0.037584)

# These samples and range have been chosen to write out this colorspace with
# a limited over/undershoot range, which also exactly samples the 0.0,1.0
# crossings

NUM_SAMPLES = 2**12+5
RANGE = (-0.125, 1.125)
data = []
for i in xrange(NUM_SAMPLES):
    x = i/(NUM_SAMPLES-1.0)
    x = Fit(x, 0.0, 1.0, RANGE[0], RANGE[1])
    data.append(fromSLog(x))

# Data is srgb->linear
WriteSPI1D('luts/slog.spi1d', RANGE[0], RANGE[1], data)

cs = OCIO.ColorSpace(name='SLog')
cs.setDescription("Sony SLog")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setAllocation(OCIO.Constants.ALLOCATION_UNIFORM)
cs.setAllocationVars([RANGE[0], RANGE[1]])

t = OCIO.FileTransform('slog.spi1d', interpolation=OCIO.Constants.INTERP_LINEAR)
cs.setTransform(t, OCIO.Constants.COLORSPACE_DIR_TO_REFERENCE)
config.addColorSpace(cs)


###############################################################################

'REDSpace'



###############################################################################

cs = OCIO.ColorSpace(name='raw')
cs.setDescription("Raw Data. Used for normals, points, etc.")
cs.setBitDepth(OCIO.Constants.BIT_DEPTH_F32)
cs.setIsData(True)
config.addColorSpace(cs)


###############################################################################

display = 'default'
config.addDisplay(display, 'None', 'raw')
config.addDisplay(display, 'sRGB', 'sRGB')
config.addDisplay(display, 'rec709', 'rec709')

config.setActiveDisplays('default')
config.setActiveViews('sRGB')


###############################################################################



try:
    config.sanityCheck()
except Exception,e:
    print e

f = file(outputfilename,"w")
f.write(config.serialize())
f.close()
print "Wrote",outputfilename

# Core/LUT/include/LUT/fnLUTConversions.h




#!/usr/bin/env python


# caspian.tif	279x220 64-bit floating point (deflate) Caspian Sea from space
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "caspian.tif")

# cramps.tif	800x607 8-bit b&w (packbits) "cramps poster"
#    This tests 1-bit images, and packbits compression
# cramps-tile.tif	256x256 tiled version of cramps.tif (no compression)
#    Tests tiled images (especially tiled 1-bit) -- compare it to cramps
# dscf0013.tif  640x480 YCbCr digital camera image which lacks Reference
#       Black/White values. Contains EXIF SubIFD. No compression.
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "cramps.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "cramps-tile.tif")
command += diff_command (OIIO_TESTSUITE_IMAGEDIR + "/cramps-tile.tif",
                         OIIO_TESTSUITE_IMAGEDIR + "/cramps.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "dscf0013.tif")

# fax2d.tif	1728x1082 1-bit b&w (G3/2D) facsimile
# FIXME - we read the pixel data fine, but we fail to recognize that
#   differing XResolution and YResolution imply a non-square pixel
#   aspect ratio, and iv fails to display it well for this reason.
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "fax2d.tif")

# g3test.tif	TIFF equivalent of g3test.g3 created by fax2tiff
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "g3test.tif")
# FIXME - same aspect ratio issue as fax2d.tif

# jello.tif	256x192 8-bit RGB (packbits palette) Paul Heckbert "jello"
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "jello.tif")

# ladoga.tif	158x118 16-bit unsigned, single band, deflate
# NOTE -- I have no idea if we read this correctly.  Neither ImageMagick
#    nor OSX preview display a meaningful image.

# off_l16.tif	333x225 8-bit CIE LogL (SGILog) office from Greg Larson
# off_luv24.tif	333x225 8-bit CIE LogLuv (SGILog24) office from " "
# off_luv32.tif	333x225	8-bit CIE LogLuv (SGILog) office from " "
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "off_l16.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "off_luv24.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "off_luv32.tif")

# pc260001.tif	640x480 8-bit RGB digital camera image. Contains EXIF SubIFD.
# 		No compression.
# FIXME? - we don't seem to recognize additional Exif data that's in the
#    'Maker Note', which includes GainControl
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "pc260001.tif")

# quad-lzw.tif	512x384 8-bit RGB (lzw) "quadric surfaces"
# quad-tile.tif	512x384 tiled version of quad-lzw.tif (lzw)
# quad-jpeg.tif 512x384 8-bit YCbCr (jpeg) version of quad-lzw.tif

# FIXME - temporarily disable quad-lzw.tif to address a regression in
# libtiff 4.0.8. Re-enable these tests after libtiff is patched.
#command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "quad-lzw.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "quad-tile.tif")
#command += diff_command (OIIO_TESTSUITE_IMAGEDIR + "/quad-tile.tif",
#                         OIIO_TESTSUITE_IMAGEDIR + "/quad-lzw.tif")
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "quad-jpeg.tif",
                       extraargs="-compression zip")

# strike.tif	256x200 8-bit RGBA (lzw) "bowling pins" from Pixar
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "strike.tif")

# text.tif	1512x359 4-bit b&w (thunderscan) am-express credit card
#  FIXME -- we don't get this right

# ycbcr-cat.tif	250x325 8-bit YCbCr (lzw) "kitty" created by rgb2ycbcr
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "ycbcr-cat.tif")

# smallliz.tif	160x160 8-bit YCbCr (OLD jpeg) lizard from HP**
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "smallliz.tif", 0)
# zackthecat.tif 234x213 8-bit YCbCr (OLD jpeg) tiled "ZackTheCat" from NeXT**
#   considered a deprecated format, not supported by libtiff
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "zackthecat.tif", 0)

# oxford.tif	601x81 8-bit RGB (lzw) screendump off oxford
command += rw_command (OIIO_TESTSUITE_IMAGEDIR, "oxford.tif", 0)

# The other images are from Hewlett Packard and exemplify the use of the
# HalftoneHints tag (in their words):
# The images are all the same subject, and should all appear the same
# after rendering.  Each of the images is slightly different as outlined
# by the following table:
#
#   FileName	   ToneRange  HalftoneHints
# jim___cg.tif      	A	    Y
# jim___dg.tif      	B	    N
# jim___gg.tif      	B	    Y
#
# NOTE -- OIIO appears to read this fine, but I'm really not sure how to
#    judge if it's "correct"



#!/usr/bin/env python

refdirlist = [
    OIIO_TESTSUITE_ROOT + "/oiiotool/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-color/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-copy/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-composite/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-fixnan/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-deep/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-pattern/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-text/ref/",
    OIIO_TESTSUITE_ROOT + "/oiiotool-xform/ref/",
    refdir
]

# Run the script
command += pythonbin + " src/test_imagebufalgo.py > out.txt ;"

# Checkout outputs -- some of the refs are in the oiiotool test dir
outputs = ["black.tif", "filled.tif", "checker.tif",
           "noise-uniform3.tif", "noise-gauss.tif", "noise-salt.tif",
           "chanshuffle.tif", "ch-rgba.exr", "ch-z.exr",
           "chappend-rgbaz.exr",
           "flat.exr", "deepen.exr",
           "crop.tif", "cut.tif", "pasted.tif",
           "rotate90.tif", "rotate180.tif", "rotate270.tif",
           "rotated.tif", "rotated-offcenter.tif",
           "warped.tif",
           "flip.tif", "flop.tif", "reorient1.tif",
           "transpose.tif",
           "cshift.tif", "cadd1.exr", "cadd2.exr", "add.exr", "sub.exr",
           "abs.exr", "absdiff.exr",
           "mul.exr", "cmul1.exr", "cmul2.exr",
           "mad.exr", "mad2.exr", "mad3.exr",
           "cpow1.exr", "cpow2.exr",
           "div.exr", "divc1.exr", "divc2.exr",
           "invert.tif",
           "chsum.tif", "colormap-inferno.tif", "colormap-custom.tif",
           "grid-clamped.tif",
           "rangecompress.tif", "rangeexpand.tif",
           "contrast-stretch.tif", "contrast-shrink.tif",
           "contrast-sigmoid5.tif",
           "resize.tif", "resample.tif", "fit.tif",
           "bsplinekernel.exr", "bspline-blur.tif", "tahoe-median.tif",
           "dilate.tif", "erode.tif",
           "unsharp.tif", "unsharp-median.tif", "tahoe-laplacian.tif",
           "fft.exr", "ifft.exr", "polar.exr", "unpolar.exr",
           "tahoe-filled.tif",
           "box3.exr",
           "a_over_b.exr",
           "tahoe-small.tx",
           "text.tif", "textcentered.tif",
           "out.txt"
         ]
    # command += checkref (f)




#!/usr/bin/env python

from __future__ import print_function
from __future__ import absolute_import
import array
import numpy
import OpenImageIO as oiio


# Print the contents of an ImageSpec
def print_imagespec (spec, subimage=0, mip=0, msg="") :
    if msg != "" :
        print (str(msg))
    if spec.depth <= 1 :
        print ("  resolution %dx%d%+d%+d" % (spec.width, spec.height, spec.x, spec.y))
    else :
        print ("  resolution %dx%d%x%d+d%+d%+d" % (spec.width, spec.height, spec.depth, spec.x, spec.y, spec.z))
    if (spec.width != spec.full_width or spec.height != spec.full_height
        or spec.depth != spec.full_depth) :
        if spec.full_depth <= 1 :
            print ("  full res   %dx%d%+d%+d" % (spec.full_width, spec.full_height, spec.full_x, spec.full_y))
        else :
            print ("  full res   %dx%d%x%d+d%+d%+d" % (spec.full_width, spec.full_height, spec.full_depth, spec.full_x, spec.full_y, spec.full_z))
    if spec.tile_width :
        print ("  tile size  %dx%dx%d" % (spec.tile_width, spec.tile_height, spec.tile_depth))
    else :
        print ("  untiled")
    if mip >= 1 :
        return
    print ("  " + str(spec.nchannels), "channels:", spec.channelnames)
    print ("  format = ", str(spec.format))
    if spec.channelformats :
        print ("  channelformats = ", spec.channelformats)
    print ("  alpha channel = ", spec.alpha_channel)
    print ("  z channel = ", spec.z_channel)
    print ("  deep = ", spec.deep)
    for attrib in spec.extra_attribs :
        if type(attrib.value) == str :
            print (" ", attrib.name, "= \"" + attrib.value + "\"")
        else :
            print (" ", attrib.name, "=", attrib.value)
    # Equivalent, using indexing rather than iterating:
    #for i in range(len(spec.extra_attribs)) :
    #    if type(spec.extra_attribs[i].value) == str :
    #        print (" ", spec.extra_attribs[i].name, "= \"" + spec.extra_attribs[i].value + "\"")
    #    else :
    #        print (" ", spec.extra_attribs[i].name, "=", spec.extra_attribs[i].value)


def write (image, filename, format=oiio.UNKNOWN) :
    if not image.has_error :
        image.write (filename, format)
    if image.has_error :
        print ("Error writing", filename, ":", image.geterror())



def test_perchannel_formats () :
    # Test writing per-channel formats with an ImageBuf
    b = oiio.ImageBuf(oiio.ImageSpec(2,2,4,"float"))
    oiio.ImageBufAlgo.fill(b, (0.1, 0.2, 0.3, 0.4))
    b.set_write_format (("half", "half", "half", "float"))
    b.write ("perchan.exr")



def test_deep () :
    # Test write and read of deep data
    # Let's try writing one
    print ("\nWriting deep buffer...")
    deepbufout_spec = oiio.ImageSpec (2, 2, 5, oiio.FLOAT)
    deepbufout_spec.channelnames = ("R", "G", "B", "A", "Z")
    deepbufout_spec.deep = True
    deepbufout = oiio.ImageBuf(deepbufout_spec)
    deepbufout.set_deep_samples (x=1, y=0, z=0, nsamples=2)
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=0, sample=0, value=0.42)
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=4, sample=0, value=42.0)
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=0, sample=1, value=0.47)
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=4, sample=1, value=43.0)
    # Also insert some new samples
    deepbufout.deep_insert_samples (x=1, y=0, z=0, samplepos=1, nsamples=2);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=0, sample=1, value=1.1);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=1, sample=1, value=2.2);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=2, sample=1, value=2.3);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=3, sample=1, value=1.0);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=3, sample=1, value=42.25);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=0, sample=2, value=0.1);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=1, sample=2, value=0.2);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=2, sample=2, value=0.3);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=3, sample=2, value=1.0);
    deepbufout.set_deep_value (x=1, y=0, z=0, channel=3, sample=2, value=42.5);
    # But delete the first one
    deepbufout.deep_erase_samples (x=1, y=0, z=0, samplepos=1, nsamples=1);
    # Save
    deepbufout.write ("deepbuf.exr")
    # And read it back
    print ("\nReading back deep buffer:")
    deepbufin = oiio.ImageBuf ("deepbuf.exr")
    deepbufin_spec = deepbufin.spec()
    dd = deepbufin.deepdata()
    for p in range(dd.pixels) :
        ns = dd.samples(p)
        if ns > 1 :
            print ("Pixel", p//deepbufin_spec.width, p%deepbufin_spec.width, "had", ns, "samples")
            for s in range(ns) :
                print ("Sample", s)
                for c in range(dd.channels) :
                    print ("\tc {0} : {1:.3f}".format(c, dd.deep_value(p,c,s)))



# Test ability to write multiple subimages to a file.
# Also tests ImageBuf.write(ImageOutput)
def test_multiimage () :
    print ("Writing multi-image file")
    spec = oiio.ImageSpec (128, 64, 3, "float")
    out = oiio.ImageOutput.create ("multipart.exr")
    # Open with intent to write two subimages, each with same spec
    if not out.open ("multipart.exr", (spec, spec)) :
        print ("Error on initial open:", out.geterror())
        return
    img = oiio.ImageBufAlgo.fill ((0.5, 0.0, 0.0), spec.roi)
    if not img.write (out) :
        print ("Error on write:", img.geterror())
        return
    if not out.open ("multipart.exr", spec, "AppendSubimage") :
        print ("Error on open for append:", out.geterror())
        return
    img = oiio.ImageBufAlgo.fill ((0.0, 0.5, 0.0), spec.roi)
    if not img.write (out) :
        print ("Error on write:", img.geterror())
        return
    out.close ()



######################################################################
# main test starts here

try:

    print ("Constructing to be a writable 320x240,4 UINT16:")
    b = oiio.ImageBuf (oiio.ImageSpec(320,240,4,oiio.UINT16))
    print_imagespec (b.spec())
    print ("Resetting to be a writable 640x480,3 Float:")
    b.reset (oiio.ImageSpec(640,480,3,oiio.FLOAT))
    print_imagespec (b.spec())
    print ("")

    # Test reading from disk
    print ("Testing read of ../common/textures/grid.tx:")
    b = oiio.ImageBuf ("../common/textures/grid.tx")
    print ("subimage:", b.subimage, " / ", b.nsubimages)
    print ("miplevel:", b.miplevel, " / ", b.nmiplevels)
    print ("channels:", b.nchannels)
    print ("name:", b.name)
    print ("file_format_name:", b.file_format_name)
    print ("deep:", b.deep)
    print ("orientation:", b.orientation)
    print ("oriented x,y,width,height:", b.oriented_x, b.oriented_y, b.oriented_width, b.oriented_height)
    print ("oriented full x,y,width,height:", b.oriented_full_x, b.oriented_full_y, b.oriented_full_width, b.oriented_full_height)
    print ("xyz beg/end:", b.xbegin, b.xend, b.ybegin, b.yend, b.zbegin, b.zend)
    print ("xyz min/max:", b.xmin, b.xmax, b.ymin, b.ymax, b.zmin, b.zmax)
    print ("setting full res...")
    b.set_full (0, 2048, 0, 2048, 0, 1)
    print ("roi =", b.roi)
    print ("full roi =", b.roi_full)
    print ("setting full roi again, as ROI...")
    b.roi_full = oiio.ROI(0, 1024, 0, 1024, 0, 1, 0, b.nchannels)
    print ("Changing origin...")
    b.set_origin (15, 20);
    print ("Printing the whole spec to be sure:")
    print_imagespec (b.spec())
    print ("")
    print ("Resetting to a different MIP level:")
    b.reset ("../common/textures/grid.tx", 0, 2)
    print_imagespec (b.spec())
    print ("")

    # Create a small buffer, do various pixel reads and writes
    print ("Making 2x2 RGB image:")
    b = oiio.ImageBuf (oiio.ImageSpec(2,2,3,oiio.UINT8))
    print_imagespec (b.spec())
    b.setpixel (0, 0, 0, (1.0, 0.0, 0.0))
    b.setpixel (1, 0, 0, (0.0, 1.0, 0.0))
    b.setpixel (0, 1, 0, (0.0, 0.0, 1.0))
    b.setpixel (1, 1, 0, (0.0, 0.0, 0.0))
    print ("Pixel 0,0 is", b.getpixel(0,0,0))
    print ("Pixel 1,0 is", b.getpixel(1,0))   # test 2D lookup
    print ("Pixel 0,1 is", b.getpixel(0,1))
    print ("Interpolating 1,0.5 ->", b.interppixel(1,0.5))
    print ("Interpolating NDC 0.25,0.5 ->", b.interppixel_NDC(0.25,0.5))
    print ("Interpolating bicubic 0.25,0.5 ->", b.interppixel_bicubic(1.0,0.5))
    print ("Interpolating NDC bicubic 0.25,0.5 ->", b.interppixel_bicubic_NDC(0.25,0.5))
    print ("The whole image is: ", b.get_pixels(oiio.TypeDesc.TypeFloat))
    print ("")
    print ("Saving file...")
    b.write ("out.tif")

    # test set_pixels, too
    b.set_pixels (oiio.ROI(0, 2, 0, 2, 0, 1, 0, 3),
                  numpy.array([[[0.1,0.0,0.9], [0.2,0.0,0.7]],
                               [[0.3,0.0,0.8], [0.4,0.0,0.6]]], dtype="f"))
    b.write ("outtuple.tif")
    b.set_pixels (oiio.ROI(0, 2, 0, 2, 0, 1, 0, 3),
                  numpy.array([0.1,0.5,0.9, 0.2,0.5,0.7, 0.3,0.5,0.8, 0.4,0.5,0.6], dtype='f'))
    b.write ("outarray.tif")
    b.set_pixels (oiio.ROI(0, 2, 0, 2, 0, 1, 0, 3),
                  numpy.array([26,128,230, 51,128,178, 76,128,204, 102,128,153], dtype='B'))
    write (b, "outarrayB.tif", oiio.UINT8)
    b.set_pixels (oiio.ROI(0, 2, 0, 2, 0, 1, 0, 3),
                  numpy.array([6554,32767,58982, 13107,32767,45874,
                                   19660,32767,52428, 26214,32767,39321], dtype='H'))
    write (b, "outarrayH.tif", oiio.UINT16)

    test_perchannel_formats ()
    test_deep ()
    test_multiimage ()

    print ("\nDone.")
except Exception as detail:
    print ("Unknown exception:", detail)




#!/usr/bin/env python

files = [ "psd_123.psd", "psd_123_nomaxcompat.psd", "psd_bitmap.psd",
          "psd_indexed_trans.psd", "psd_rgb_8.psd", "psd_rgb_16.psd",
          "psd_rgb_32.psd", "psd_rgba_8.psd" ]
for f in files:
    command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/" + f)

command += info_command ("src/different-mask-size.psd")
command += info_command ("src/layer-mask.psd")



#!/usr/bin/env python

# Make sure we can do a read-write-read round trip of all the OpenEXR
# test images that exercise different display and pixel windows.


# ../openexr-images/DisplayWindow:
# README   t03.exr  t06.exr  t09.exr  t12.exr  t15.exr
# t01.exr  t04.exr  t07.exr  t10.exr  t13.exr  t16.exr
# t02.exr  t05.exr  t08.exr  t11.exr  t14.exr
imagedir = OIIO_TESTSUITE_IMAGEDIR + "/DisplayWindow"
files = [
    "t01.exr", "t02.exr", "t03.exr", "t04.exr", "t05.exr", "t06.exr",
    "t07.exr", "t08.exr", "t09.exr", "t10.exr", "t11.exr", "t12.exr",
    "t13.exr", "t14.exr", "t15.exr", "t16.exr"
]
for f in files:
    command += rw_command (imagedir, f)




#!/usr/bin/env python

imagedir = "ref/"
files = [ "IMG_7702_small.heic", "Chimera-AV1-8bit-162.avif" ]
for f in files:
    command = command + info_command (os.path.join(imagedir, f))

# avif conversion is expected to fail if libheif is built without AV1 support
failureok = 1



#!/usr/bin/env python

imagedir = "../texture-field3d/src/"
files = [ "dense_float.f3d", "sparse_float.f3d",
          "dense_half.f3d", "sparse_half.f3d" ]

for f in files:
    command += rw_command (imagedir, f)
